﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using I7565CPM_Net;

namespace SDO_Read
{
    public partial class Form1 : Form
    {
        byte COMPORT;
        byte BAUD;
        byte NODE;

        byte BlockMode = 1;

        ushort DelayTime;
        ushort ret;
        string errstr;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            COMPORT = 0;
            BAUD = 0;
            NODE = 1;

            Combo_Com.Enabled = true;
            Combo_Baud.Enabled = true;
            Combo_Node.Enabled = false;

            Combo_Com.SelectedIndex = 0;
            Combo_Baud.SelectedIndex = 0;
            Combo_Node.SelectedIndex = 0;

            Btn_AddNode.Enabled = false;
            Btn_AddNode.Visible = true;
            Btn_Remove.Enabled = false;

            Pal_SDO.Enabled = false;
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            I7565CPM.I7565CPM_ShutdownMaster(COMPORT);
        }

        private void Combo_Com_SelectedIndexChanged(object sender, EventArgs e)
        {
            COMPORT = (byte)(Combo_Com.SelectedIndex);
        }

        private void Combo_Baud_SelectedIndexChanged(object sender, EventArgs e)
        {
            BAUD = (byte)(Combo_Baud.SelectedIndex);

            switch (BAUD)
            {
                case 0:
                    DelayTime = 15;
                    break;
                case 1:
                    DelayTime = 7;
                    break;
                case 2:
                    DelayTime = 3;
                    break;
                case 3:
                case 4:
                    DelayTime = 1;
                    break;
                case 5:
                case 6:
                case 7:
                    DelayTime = 0;
                    break;
            }
        }

        private void Combo_Node_SelectedIndexChanged(object sender, EventArgs e)
        {
            NODE = (byte)(Combo_Node.SelectedIndex + 1);
        }

        private void Btn_Initial_Click(object sender, EventArgs e)
        {
            ret = I7565CPM.I7565CPM_InitMaster(COMPORT, 0, BAUD, BlockMode);
            if (ret != 0)
            {
                errstr = "Initial Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Combo_Com.Enabled = false;
                Combo_Baud.Enabled = false;
                Combo_Node.Enabled = true;

                Btn_Initial.Enabled = false;
                Btn_Initial.Visible = false;
                Btn_Shutdown.Enabled = true;

                Btn_AddNode.Enabled = true;
                Btn_AddNode.Visible = true;
            }
        }

        private void Btn_Shutdown_Click(object sender, EventArgs e)
        {
            ret = I7565CPM.I7565CPM_ShutdownMaster(COMPORT);
            if (ret != 0)
            {
                errstr = "Shutdown Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Combo_Com.Enabled = true;
                Combo_Baud.Enabled = true;
                Combo_Node.Enabled = false;

                Btn_Initial.Enabled = true;
                Btn_Initial.Visible = true;
                Btn_Shutdown.Enabled = false;

                Btn_AddNode.Enabled = false;
                Btn_AddNode.Visible = true;
                Btn_Remove.Enabled = false;

                Pal_SDO.Enabled = false;
            }
        }

        private void Btn_AddNode_Click(object sender, EventArgs e)
        {
            ret = I7565CPM.I7565CPM_AddNode(COMPORT, NODE, 1, DelayTime, 200, BlockMode);
            if (ret != 0)
            {
                errstr = "AddNode Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Btn_AddNode.Enabled = false;
                Btn_AddNode.Visible = false;
                Btn_Remove.Enabled = true;
                Combo_Node.Enabled = false;

                Pal_SDO.Enabled = true;
            }
        }

        private void Btn_Remove_Click(object sender, EventArgs e)
        {
            ret = I7565CPM.I7565CPM_RemoveNode(COMPORT, NODE, BlockMode);
            if (ret != 0)
            {
                errstr = "RemoveNode Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Btn_AddNode.Enabled = true;
                Btn_AddNode.Visible = true;
                Btn_Remove.Enabled = false;
                Combo_Node.Enabled = true;

                Pal_SDO.Enabled = false;
            }
        }

        private void Btn_Clear_Click(object sender, EventArgs e)
        {
            List_SDOData.Items.Clear();
        }

        private void Check_Is_Hex(object sender, KeyPressEventArgs e)
        {
            if (!(("1234567890ABCDEFabcdef".IndexOf(e.KeyChar) != -1) || ((int)e.KeyChar == 8))) e.Handled = true;
        }

        private void Btn_ReadSDO_Click(object sender, EventArgs e)
        {
            ushort Index;
            uint i, RDlen;
            byte SubIndex;
            byte[] RData = new byte[1024];
            string DataStr;

            Index = Convert.ToUInt16(Text_Index.Text, 16);

            SubIndex = Convert.ToByte(Text_SubIndex.Text, 16);

            ret = I7565CPM.I7565CPM_SDOReadData(COMPORT, NODE, Index, SubIndex, out RDlen, RData, BlockMode);
            if ((ret != 0) && (ret != I7565CPM._25_CPM_Abort))
            {
                errstr = "SDO Read Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                DataStr = Index.ToString("X4") + "-" + SubIndex.ToString("X2") + ": ";

                for (i = 0; i < RDlen; i++)
                {
                    DataStr = DataStr + RData[i].ToString("X2") + " ";
                }

                if (ret == I7565CPM._25_CPM_Abort)
                {
                    DataStr = DataStr + ": Read Abort";
                }
                List_SDOData.Items.Add(DataStr);

            }
        }

    }
}