﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using I7565CPM_Net;

namespace Scan_Node
{
    public partial class Form1 : Form
    {
        byte COMPORT;
        byte BAUD;
        byte NODE;

        byte BlockMode = 1;

        byte NODE_CNT = 0;

        ushort DelayTime;
        ushort ret;
        string errstr;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            COMPORT = 0;
            BAUD = 0;
            NODE = 0;

            Combo_Com.Enabled = true;
            Combo_Baud.Enabled = true;
            Combo_Node.Enabled = false;

            Combo_Com.SelectedIndex = 0;
            Combo_Baud.SelectedIndex = 0;

            Btn_AddNode.Enabled = false;
            Btn_Remove.Enabled = false;
            Btn_ScanNode.Enabled = false;
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            I7565CPM.I7565CPM_ShutdownMaster(COMPORT);
        }

        private void Combo_Com_SelectedIndexChanged(object sender, EventArgs e)
        {
            COMPORT = (byte)(Combo_Com.SelectedIndex);
        }

        private void Combo_Baud_SelectedIndexChanged(object sender, EventArgs e)
        {
            BAUD = (byte)(Combo_Baud.SelectedIndex);

            switch (BAUD)
            {
                case 0:
                    DelayTime = 15;
                    break;
                case 1:
                    DelayTime = 7;
                    break;
                case 2:
                    DelayTime = 3;
                    break;
                case 3:
                case 4:
                    DelayTime = 1;
                    break;
                case 5:
                case 6:
                case 7:
                    DelayTime = 0;
                    break;
            }
        }

        private void Combo_Node_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Convert text to Node ID number
            if (NODE_CNT != 0) NODE = Convert.ToByte(Combo_Node.Text, 10);
        }

        private void Btn_Initial_Click(object sender, EventArgs e)
        {
            ret = I7565CPM.I7565CPM_InitMaster(COMPORT, 0, BAUD, BlockMode);
            if (ret != 0)
            {
                errstr = "Initial Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Combo_Com.Enabled = false;
                Combo_Baud.Enabled = false;
                Combo_Node.Enabled = true;

                Btn_Initial.Enabled = false;
                Btn_Initial.Visible = false;
                Btn_Shutdown.Enabled = true;

                Btn_AddNode.Enabled = true;
                Btn_Remove.Enabled = true;
                Btn_ScanNode.Enabled = true;
            }
        }

        private void Btn_Shutdown_Click(object sender, EventArgs e)
        {
            ret = I7565CPM.I7565CPM_ShutdownMaster(COMPORT);
            if (ret != 0)
            {
                errstr = "Shutdown Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Combo_Com.Enabled = true;
                Combo_Baud.Enabled = true;
                Combo_Node.Enabled = false;

                Btn_Initial.Enabled = true;
                Btn_Initial.Visible = true;
                Btn_Shutdown.Enabled = false;

                Btn_AddNode.Enabled = false;
                Btn_Remove.Enabled = false;
                Btn_ScanNode.Enabled = false;
            }
        }

        private void Btn_AddNode_Click(object sender, EventArgs e)
        {
            ret = I7565CPM.I7565CPM_AddNode(COMPORT, NODE, 1, DelayTime, 200, BlockMode);
            if (ret != 0)
            {
                errstr = "AddNode Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                byte[] NodeList = new byte[16];
                byte Temp_Node = 0;
                ushort i, j, Flag = 0;
                string NodeNumber;

                ret = I7565CPM.I7565CPM_GetNodeList(COMPORT, NodeList, BlockMode);
                if (ret != 0)
                {
                    errstr = "Node List Get Error, = " + ret.ToString("D");
                    MessageBox.Show(errstr);
                }
                else
                {
                    List_NodeList.Items.Clear();
                    // NodeList[0] is node 0 ~ 7
                    // NodeList[1] is node 8 ~ 15
                    // ...
                    // NodeList[15] is node 120 ~ 127

                    for (i = 0; i < 16; i++)
                    {
                        Flag = 1;

                        for (j = 0; j < 8; j++)
                        {
                            if ((NodeList[i] & Flag) != 0)
                            {
                                NodeNumber = "Node " + Temp_Node.ToString("D3") + " under controll";
                                List_NodeList.Items.Add(NodeNumber);
                            }

                            Temp_Node = (byte)(Temp_Node + 1);
                            Flag = (ushort)(Flag * 2);
                        }
                    }
                }
            }
        }

        private void Btn_Remove_Click(object sender, EventArgs e)
        {
            ret = I7565CPM.I7565CPM_RemoveNode(COMPORT, NODE, BlockMode);
            if (ret != 0)
            {
                errstr = "RemoveNode Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                byte[] NodeList = new byte[16];
                byte Temp_Node = 0;
                ushort i, j, Flag = 0;
                string NodeNumber;

                ret = I7565CPM.I7565CPM_GetNodeList(COMPORT, NodeList, BlockMode);
                if (ret != 0)
                {
                    errstr = "Node List Get Error, = " + ret.ToString("D");
                    MessageBox.Show(errstr);
                }
                else
                {
                    List_NodeList.Items.Clear();
                    // NodeList[0] is node 0 ~ 7
                    // NodeList[1] is node 8 ~ 15
                    // ...
                    // NodeList[15] is node 120 ~ 127

                    for (i = 0; i < 16; i++)
                    {
                        Flag = 1;

                        for (j = 0; j < 8; j++)
                        {
                            if ((NodeList[i] & Flag) != 0)
                            {
                                NodeNumber = "Node " + Temp_Node.ToString("D3") + " under controll";
                                List_NodeList.Items.Add(NodeNumber);
                            }

                            Temp_Node = (byte)(Temp_Node + 1);
                            Flag = (ushort)(Flag * 2);
                        }
                    }
                }
            }
        }

        private void Btn_ScanNode_Click(object sender, EventArgs e)
        {
            byte[] NodeList = new byte[16];
            byte Temp_Node = 0;
            ushort i, j, Flag = 0;
            string NodeNumber;

            NODE_CNT = 0;

            // Set function timeout to 3000 ms
            I7565CPM.I7565CPM_SetFunctionTimeout(COMPORT, 5000);

            // Scan total node to check witch nodes are on the CANopen bus
            ret = I7565CPM.I7565CPM_ScanNode(COMPORT, 0, 0, NodeList, BlockMode);
            if (ret != 0)
            {
                errstr = "Node Scan Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                // Set Function Timeout to 1000 ms (default value)
                I7565CPM.I7565CPM_SetFunctionTimeout(COMPORT, 3000);

                Combo_Node.Items.Clear();
                // NodeList[0] is node 0 ~ 7
                // NodeList[1] is node 8 ~ 15
                // ...
                // NodeList[15] is node 120 ~ 127

                for (i = 0; i < 16; i++)
                {
                    Flag = 1;

                    for (j = 0; j < 8; j++)
                    {
                        if ((NodeList[i] & Flag) != 0)
                        {
                            NodeNumber = Temp_Node.ToString("D3");
                            Combo_Node.Items.Add(NodeNumber);
                            NODE_CNT++;
                        }

                        Temp_Node = (byte)(Temp_Node + 1);
                        Flag = (ushort)(Flag * 2);
                    }
                }

                if (Combo_Node.Items.Count != 0)
                {
                    Combo_Node.Enabled = true;
                    Btn_AddNode.Enabled = true;
                    Btn_Remove.Enabled = true;
                }
                else
                {
                    Combo_Node.Items.Add("None Slave!");

                    Combo_Node.Enabled = false;
                    Btn_AddNode.Enabled = false;
                    Btn_Remove.Enabled = false;

                    NODE = 0;
                }

                Combo_Node.SelectedIndex = 0;
            }
        }
    }
}